/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  28.10.2016  mifi  First Version, tested with an Embedded Artists 
*                    LPC4088 Quickstart Board and QSB Base Board.
**************************************************************************/
#if defined(USE_BOARD_QSB4088)
#define __TALIO_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_IOInit                                                           */
/*                                                                       */
/*  Initialize the IOs of the EA-1788 board.                             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_IOInit (void)
{
   /* Turn on GPIO clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);

   /*
    * Power off the GPIOs of the SC card interface
    */
   
   /* First disable the power by the MOSFET */ 
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 5, (IOCON_FUNC0 | IOCON_DIGMODE_EN));
   Chip_GPIO_SetPinDIROutput(LPC_GPIO, 1, 5);
   LPC_GPIO1->SET = (1<<5);   /* Power off */

   /* Set all SPI pins to output, with PullDown */   
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 2,  (IOCON_FUNC0 | IOCON_MODE_PULLDOWN));
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 3,  (IOCON_FUNC0 | IOCON_MODE_PULLDOWN));
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 6,  (IOCON_FUNC0 | IOCON_MODE_PULLDOWN | IOCON_DIGMODE_EN));
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 7,  (IOCON_FUNC0 | IOCON_MODE_PULLDOWN | IOCON_DIGMODE_EN));
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 11, (IOCON_FUNC0 | IOCON_MODE_PULLDOWN));
   Chip_IOCON_PinMuxSet(LPC_IOCON, 1, 12, (IOCON_FUNC0 | IOCON_MODE_PULLDOWN));
   
   /* Ouput low signal on all SPI pins */
   LPC_GPIO1->CLR = ((1<<2) | (1<<3) | (1<<6) | (1<<7) | (1<<11) | (1<<12));
   
} /* tal_IOInit */

#endif /* USE_BOARD_QSB4088 */

/*** EOF ***/
