/******************************************************************************
  Target Script for LPC177x/LPC178x

  Copyright (c) 2012 Rowley Associates Limited.

  Partial Copyright (c) 2013, Michael Fischer
  Added functionality of PLLInit, SDRAMInit and MPUInit.

  This file may be distributed under the terms of the License Agreement
  provided with this software.

  THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ******************************************************************************/

function crcIsOK()
{
  crc = 0;
  for (i = 0x00; i < 0x1C; i += 4)
    crc += TargetInterface.peekUint32(i);
  return crc == ~TargetInterface.peekUint32(0x1C) + 1;
}

function Reset()
{
  var impl = TargetInterface.implementation ? TargetInterface.implementation() : "";
  if (impl == "j-link")
    TargetInterface.resetAndStop(100);
  else if (impl != "crossworks_simulator")
    {
      /* Carry out system reset */
      TargetInterface.pokeWord(0xE000EDFC, 0x00000001);
      TargetInterface.pokeWord(0xE000ED0C, 0x05FA0004);
      /* Run the boot loader initialization code */
      TargetInterface.pokeWord(0xE000EDFC, 0x01000000);
      TargetInterface.pokeWord(0xE0001020, 0x00000000);
      TargetInterface.pokeWord(0xE0001024, 0x00000000);
      TargetInterface.pokeWord(0xE0001028, 0x00000005);
      TargetInterface.go();
      TargetInterface.delay(100); 
      TargetInterface.resetDebugInterface();
      TargetInterface.waitForDebugState(1000);
      if (crcIsOK())
        {
          TargetInterface.go();
          TargetInterface.waitForDebugState(1000);
        }
      TargetInterface.pokeWord(0xE0001028, 0x00000000);
    }
}

function PLLInit()
{
  /* 
   * Setup Clock: 
   * XTAL = 12 MHz 
   *   => CPU = 120 MHz, 
   *   => Peripheral = 60 MHz
   *   => EMC = 60 MHz
   *   => USB = 48 MHz
   */

  /* Enable main OSC */
  TargetInterface.pokeWord(0x400FC1A0, 0x00000021);   /* SCS */
  
  /* Wait until main OSC is usable */
  TargetInterface.delay(100);                         
  
  /* Select Clock Source for sysclk/PLL0*/
  TargetInterface.pokeWord(0x400FC10C, 0x00000001);   /* CLKSRCSEL */
  
  /* Setup PLL0 */
  TargetInterface.pokeWord(0x400FC084, 0x00000009);   /* PLL0CFG */
  TargetInterface.pokeWord(0x400FC080, 0x00000001);   /* PLL0CON */
  TargetInterface.pokeWord(0x400FC08C, 0x000000AA);   /* PLL0FEED */
  TargetInterface.pokeWord(0x400FC08C, 0x00000055);   /* PLL0FEED */
  
  /* Wait for PLOCK0  */   
  TargetInterface.delay(100);                         
  
  /* Setup PLL1 */
  TargetInterface.pokeWord(0x400FC0A4, 0x00000023);   /* PLL1CFG */
  TargetInterface.pokeWord(0x400FC0A0, 0x00000001);   /* PLL1CON */
  TargetInterface.pokeWord(0x400FC0AC, 0x000000AA);   /* PLL1FEED */
  TargetInterface.pokeWord(0x400FC0AC, 0x00000055);   /* PLL1FEED */
  
  /* Wait for PLOCK1  */   
  TargetInterface.delay(100);                         

  TargetInterface.pokeWord(0x400FC104, 0x00000101);   /* Setup Clock Divider */   
  TargetInterface.pokeWord(0x400FC108, 0x00000201);   /* Setup USB Clock Divider */   
  TargetInterface.pokeWord(0x400FC100, 0x00000001);   /* EMC Clock Selection */   
  TargetInterface.pokeWord(0x400FC1A8, 0x00000002);   /* Peripheral Clock Selection */   
  TargetInterface.pokeWord(0x400FC0C4, 0x042887DE);   /* Power Control for Peripherals */   
  TargetInterface.pokeWord(0x400FC1C8, 0x00000100);   /* Clock Output Configuration */   

  TargetInterface.pokeWord(0x400FC1B0, 0x00000003);   /* Power Boost control */   

  TargetInterface.pokeWord(0x400FC000, 0x0000503A);   /* Flash Accelerator Setup */   
  
  TargetInterface.pokeWord(0xE000ED08, 0x10000000);   /* Setup vector in RAM */   
}

function SDRAMInit()
{
  TargetInterface.pokeWord(0x400FC0C4, 0x04288FDE);   /* Power Control for External Memory Controller */  
  
  TargetInterface.pokeWord(0x400FC1DC, 0x00001010);   /* EMCDLYCTL */
  TargetInterface.pokeWord(0x2009C000, 0x00000001);   /* LPC_EMC->Control */
  TargetInterface.pokeWord(0x2009C008, 0x00000000);   /* LPC_EMC->Config */

  TargetInterface.pokeWord(0x4002C180, 0x00000001);   /* LPC_IOCON->P3_0  / D0  */
  TargetInterface.pokeWord(0x4002C184, 0x00000001);   /* LPC_IOCON->P3_1  / D1  */
  TargetInterface.pokeWord(0x4002C188, 0x00000001);   /* LPC_IOCON->P3_2  / D2  */
  TargetInterface.pokeWord(0x4002C18C, 0x00000001);   /* LPC_IOCON->P3_3  / D3  */
  TargetInterface.pokeWord(0x4002C190, 0x00000001);   /* LPC_IOCON->P3_4  / D4  */
  TargetInterface.pokeWord(0x4002C194, 0x00000001);   /* LPC_IOCON->P3_5  / D5  */
  TargetInterface.pokeWord(0x4002C198, 0x00000001);   /* LPC_IOCON->P3_6  / D6  */
  TargetInterface.pokeWord(0x4002C19C, 0x00000001);   /* LPC_IOCON->P3_7  / D7  */
  TargetInterface.pokeWord(0x4002C1A0, 0x00000001);   /* LPC_IOCON->P3_8  / D8  */
  TargetInterface.pokeWord(0x4002C1A4, 0x00000001);   /* LPC_IOCON->P3_9  / D9  */
  TargetInterface.pokeWord(0x4002C1A8, 0x00000001);   /* LPC_IOCON->P3_10 / D10 */
  TargetInterface.pokeWord(0x4002C1AC, 0x00000001);   /* LPC_IOCON->P3_11 / D11 */
  TargetInterface.pokeWord(0x4002C1B0, 0x00000001);   /* LPC_IOCON->P3_12 / D12 */
  TargetInterface.pokeWord(0x4002C1B4, 0x00000001);   /* LPC_IOCON->P3_13 / D13 */
  TargetInterface.pokeWord(0x4002C1B8, 0x00000001);   /* LPC_IOCON->P3_14 / D14 */
  TargetInterface.pokeWord(0x4002C1BC, 0x00000001);   /* LPC_IOCON->P3_15 / D15 */
  TargetInterface.pokeWord(0x4002C1C0, 0x00000001);   /* LPC_IOCON->P3_16 / D16 */
  TargetInterface.pokeWord(0x4002C1C4, 0x00000001);   /* LPC_IOCON->P3_17 / D17 */
  TargetInterface.pokeWord(0x4002C1C8, 0x00000001);   /* LPC_IOCON->P3_18 / D18 */
  TargetInterface.pokeWord(0x4002C1CC, 0x00000001);   /* LPC_IOCON->P3_19 / D19 */
  TargetInterface.pokeWord(0x4002C1D0, 0x00000001);   /* LPC_IOCON->P3_20 / D20 */
  TargetInterface.pokeWord(0x4002C1D4, 0x00000001);   /* LPC_IOCON->P3_21 / D21 */
  TargetInterface.pokeWord(0x4002C1D8, 0x00000001);   /* LPC_IOCON->P3_22 / D22 */
  TargetInterface.pokeWord(0x4002C1DC, 0x00000001);   /* LPC_IOCON->P3_23 / D23 */
  TargetInterface.pokeWord(0x4002C1E0, 0x00000001);   /* LPC_IOCON->P3_24 / D24 */
  TargetInterface.pokeWord(0x4002C1E4, 0x00000001);   /* LPC_IOCON->P3_25 / D25 */
  TargetInterface.pokeWord(0x4002C1E8, 0x00000001);   /* LPC_IOCON->P3_26 / D26 */
  TargetInterface.pokeWord(0x4002C1EC, 0x00000001);   /* LPC_IOCON->P3_27 / D27 */
  TargetInterface.pokeWord(0x4002C1F0, 0x00000001);   /* LPC_IOCON->P3_28 / D28 */
  TargetInterface.pokeWord(0x4002C1F4, 0x00000001);   /* LPC_IOCON->P3_29 / D29 */
  TargetInterface.pokeWord(0x4002C1F8, 0x00000001);   /* LPC_IOCON->P3_30 / D30 */
  TargetInterface.pokeWord(0x4002C1FC, 0x00000001);   /* LPC_IOCON->P3_31 / D31 */

  TargetInterface.pokeWord(0x4002C200, 0x00000001);   /* LPC_IOCON->P4_0  / A0  */
  TargetInterface.pokeWord(0x4002C204, 0x00000001);   /* LPC_IOCON->P4_1  / A1  */
  TargetInterface.pokeWord(0x4002C208, 0x00000001);   /* LPC_IOCON->P4_2  / A2  */
  TargetInterface.pokeWord(0x4002C20C, 0x00000001);   /* LPC_IOCON->P4_3  / A3  */
  TargetInterface.pokeWord(0x4002C210, 0x00000001);   /* LPC_IOCON->P4_4  / A4  */
  TargetInterface.pokeWord(0x4002C214, 0x00000001);   /* LPC_IOCON->P4_5  / A5  */
  TargetInterface.pokeWord(0x4002C218, 0x00000001);   /* LPC_IOCON->P4_6  / A6  */
  TargetInterface.pokeWord(0x4002C21C, 0x00000001);   /* LPC_IOCON->P4_7  / A7  */
  TargetInterface.pokeWord(0x4002C220, 0x00000001);   /* LPC_IOCON->P4_8  / A8  */
  TargetInterface.pokeWord(0x4002C224, 0x00000001);   /* LPC_IOCON->P4_9  / A9  */
  TargetInterface.pokeWord(0x4002C228, 0x00000001);   /* LPC_IOCON->P4_10 / A10 */
  TargetInterface.pokeWord(0x4002C22C, 0x00000001);   /* LPC_IOCON->P4_11 / A11 */
  TargetInterface.pokeWord(0x4002C230, 0x00000001);   /* LPC_IOCON->P4_12 / A12 */
  TargetInterface.pokeWord(0x4002C234, 0x00000001);   /* LPC_IOCON->P4_13 / A13 */
  TargetInterface.pokeWord(0x4002C238, 0x00000001);   /* LPC_IOCON->P4_14 / A14 */
  TargetInterface.pokeWord(0x4002C23C, 0x00000001);   /* LPC_IOCON->P4_15 / A15 */
  TargetInterface.pokeWord(0x4002C240, 0x00000001);   /* LPC_IOCON->P4_16 / A16 */
  TargetInterface.pokeWord(0x4002C244, 0x00000001);   /* LPC_IOCON->P4_17 / A17 */
  TargetInterface.pokeWord(0x4002C248, 0x00000001);   /* LPC_IOCON->P4_18 / A18 */
  TargetInterface.pokeWord(0x4002C24C, 0x00000001);   /* LPC_IOCON->P4_19 / A19 */
  TargetInterface.pokeWord(0x4002C250, 0x00000001);   /* LPC_IOCON->P4_20 / A20 */
  TargetInterface.pokeWord(0x4002C254, 0x00000001);   /* LPC_IOCON->P4_21 / A21 */
  TargetInterface.pokeWord(0x4002C258, 0x00000001);   /* LPC_IOCON->P4_22 / A22 */
  TargetInterface.pokeWord(0x4002C25C, 0x00000001);   /* LPC_IOCON->P4_23 / A23 */
  
  TargetInterface.pokeWord(0x4002C260, 0x00000001);   /* LPC_IOCON->P4_24 / OEN */
  TargetInterface.pokeWord(0x4002C264, 0x00000001);   /* LPC_IOCON->P4_25 / WEN */
  TargetInterface.pokeWord(0x4002C268, 0x00000001);   /* LPC_IOCON->P4_26 / BLSN[0] */
  TargetInterface.pokeWord(0x4002C26C, 0x00000001);   /* LPC_IOCON->P4_27 / BLSN[1] */
  TargetInterface.pokeWord(0x4002C270, 0x00000001);   /* LPC_IOCON->P4_28 / BLSN[2] */
  TargetInterface.pokeWord(0x4002C274, 0x00000001);   /* LPC_IOCON->P4_29 / BLSN[3] */
  TargetInterface.pokeWord(0x4002C278, 0x00000001);   /* LPC_IOCON->P4_30 / CSN[0] */
  TargetInterface.pokeWord(0x4002C27C, 0x00000001);   /* LPC_IOCON->P4_31 / CSN[1] */
  TargetInterface.pokeWord(0x4002C138, 0x00000001);   /* LPC_IOCON->P2_14 / CSN[2] */
  TargetInterface.pokeWord(0x4002C13C, 0x00000001);   /* LPC_IOCON->P2_15 / CSN[3] */
  TargetInterface.pokeWord(0x4002C140, 0x00000001);   /* LPC_IOCON->P2_16 / CASN */
  TargetInterface.pokeWord(0x4002C144, 0x00000001);   /* LPC_IOCON->P2_17 / RASN */
  TargetInterface.pokeWord(0x4002C148, 0x00000001);   /* LPC_IOCON->P2_18 / CLK[0] */
  TargetInterface.pokeWord(0x4002C14C, 0x00000001);   /* LPC_IOCON->P2_19 / CLK[1] */
  TargetInterface.pokeWord(0x4002C150, 0x00000001);   /* LPC_IOCON->P2_20 / DYCSN[0] */
  TargetInterface.pokeWord(0x4002C154, 0x00000001);   /* LPC_IOCON->P2_21 / DYCSN[1]*/
  TargetInterface.pokeWord(0x4002C158, 0x00000001);   /* LPC_IOCON->P2_22 / DYCSN[2] */
  TargetInterface.pokeWord(0x4002C15C, 0x00000001);   /* LPC_IOCON->P2_23 / DYCSN[3] */
  TargetInterface.pokeWord(0x4002C160, 0x00000001);   /* LPC_IOCON->P2_24 / CKE[0] */
  TargetInterface.pokeWord(0x4002C164, 0x00000001);   /* LPC_IOCON->P2_25 / CKE[1]*/
  /* TargetInterface.pokeWord(0x4002C168, 0x00000001);   LPC_IOCON->P2_26 / CKE[2] */
  /* TargetInterface.pokeWord(0x4002C16C, 0x00000001);   LPC_IOCON->P2_27 / CKE[3] */
  TargetInterface.pokeWord(0x4002C170, 0x00000001);   /* LPC_IOCON->P2_28 / DQM[0] */
  TargetInterface.pokeWord(0x4002C174, 0x00000001);   /* LPC_IOCON->P2_29 / DQM[1]*/
  TargetInterface.pokeWord(0x4002C178, 0x00000001);   /* LPC_IOCON->P2_30 / DQM[2] */
  TargetInterface.pokeWord(0x4002C17C, 0x00000001);   /* LPC_IOCON->P2_31 / DQM[3] */

  /*
   * Configure SDRAM timings for a 60MHz bus 
   */  
  
  TargetInterface.pokeWord(0x2009C100, 0x00004480);   /* DynamicConfig0 */
  TargetInterface.pokeWord(0x2009C104, 0x00000202);   /* DynamicRasCas0 */
  TargetInterface.pokeWord(0x2009C028, 0x00000001);   /* DynamicReadConfig */
  TargetInterface.pokeWord(0x2009C030, 0x00000001);   /* DynamicRP */
  TargetInterface.pokeWord(0x2009C034, 0x00000003);   /* DynamicRAS */
  TargetInterface.pokeWord(0x2009C038, 0x00000005);   /* DynamicSREX */
  TargetInterface.pokeWord(0x2009C03C, 0x00000002);   /* DynamicAPR */
  TargetInterface.pokeWord(0x2009C040, 0x00000003);   /* DynamicDAL */
  TargetInterface.pokeWord(0x2009C044, 0x00000001);   /* DynamicWR */
  TargetInterface.pokeWord(0x2009C048, 0x00000004);   /* DynamicRC */
  TargetInterface.pokeWord(0x2009C04C, 0x00000004);   /* DynamicRFC */
  TargetInterface.pokeWord(0x2009C050, 0x00000005);   /* DynamicXSR */
  TargetInterface.pokeWord(0x2009C054, 0x00000001);   /* DynamicRRD */
  TargetInterface.pokeWord(0x2009C058, 0x00000001);   /* DynamicMRD */
  TargetInterface.delay(100);                         /* Wait 100 ms */
  TargetInterface.pokeWord(0x2009C020, 0x00000183);   /* DynamicControl */
  TargetInterface.delay(200);                         /* Wait 200 ms */
  TargetInterface.pokeWord(0x2009C020, 0x00000103);   /* DynamicControl */
  TargetInterface.pokeWord(0x2009C024, 0x00000002);   /* DynamicRefresh */
  TargetInterface.delay(1);                           /* Wait 1 ms */
  TargetInterface.pokeWord(0x2009C024, 0x0000003A);   /* DynamicRefresh */
  TargetInterface.pokeWord(0x2009C020, 0x00000083);   /* DynamicControl */
  
  TargetInterface.peekWord(0xA0044000);   /* Dummy Read */
   
  TargetInterface.pokeWord(0x2009C020, 0x00000000);   /* DynamicControl */
  TargetInterface.pokeWord(0x2009C100, 0x00084480);   /* DynamicConfig0 */
  TargetInterface.delay(1);                           /* Wait 1 ms */

  TargetInterface.pokeWord(0x400FC1DC, 0x00001111);   /* EMCDLYCTL */   
}

function MPUInit()
{
  /*
   * For more information about the MPU, take a look at:
   *
   * The definitive guide to the ARM Cortex-M3.
   * Chapter 13, The Memory Protection Unit.
   */
      
  /* MPU Region Number */
  TargetInterface.pokeWord(0xE000ED98, 0x00000000);   /* MPU->RNR  */  
  
  /* MPU Region Base Address */
  TargetInterface.pokeWord(0xE000ED9C, 0xA0000000);   /* MPU->RBAR */  
  
  /* MPU Region Attribute and Size */
  /* Full access and Region Size of 32 MB */
  TargetInterface.pokeWord(0xE000EDA0, 0x03000031);   /* MPU->RASR */  
  
  /* MPU Control register, here all options are enabled */
  TargetInterface.pokeWord(0xE000ED94, 7);
}

function SRAMReset()
{
  Reset();
  PLLInit();
  SDRAMInit();
  MPUInit();
}

function FLASHReset()
{
  Reset();
}

